﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;


//---------------------------------------------------------------------------------------------------
public class CustomerBehaviour : MonoBehaviour
{
    public Material goodHandMaterial;
    public GameObject[] hands;
    public GameObject npc_1;
    public GameObject npc_2;

	public eDrinkType m_request;
	public eDrinkType m_given;
	public bool m_hasDrink = false;

	public float m_rating = 5.0f;
	public float m_timeUntilNextStar = 3.5f;
	public float m_waitTimer = 0.0f;

	public eCustomerState m_state = eCustomerState.NONE;
	public Transform m_startSpot;
	public Transform m_targetSpot;
	public int m_spotIndex = -1;
	float m_moveTimer = 0.0f;
	float m_speed = 5.0f;

	public GameObject[] m_stars = new GameObject[5];


	//------------------------------------------------------------
	void Start()
	{
		ChangeState(eCustomerState.IN_LINE);
		m_request = GetRandomDrink();

        if(Random.Range(0,2) == 0)
        {
            npc_1.SetActive(true);
        }
        else
        {
            npc_2.SetActive(true);
        }
    }


	//---------------------------------------------------------------------------------------------------
	void EnterState(eCustomerState state)
	{
        switch (state)
		{
			case eCustomerState.IN_LINE:
				m_moveTimer = 0.0f;
				break;
			case eCustomerState.LEAVING:
                UpdateStars();
                GameManager.instance.AddScore(m_rating);
                string content = GameManager.instance.GetComponent<ReviewManager>().selectReview((int)Mathf.Round(m_rating));
                GameManager.instance.StartPopup(content);
                m_moveTimer = 0.0f;
				break;
		}
	}


	//------------------------------------------------------------
	//UpdateState
	void Update()
	{
		switch (m_state)
		{
			case eCustomerState.IN_LINE:
				InLine();
				break;
			case eCustomerState.ORDERING:
				Ordering();
				if (Input.GetKeyDown(KeyCode.Space))
				{
					ChangeState(eCustomerState.LEAVING);
				}
				break;
			case eCustomerState.LEAVING:
				Leaving();
				break;
		}
	}


	//---------------------------------------------------------------------------------------------------
	void ExitState( eCustomerState state )
	{
		switch(state)
		{
			case eCustomerState.ORDERING:
				CustomerSpawner.instance.FreeSpot(m_spotIndex);
				break;
		}
	}


	//---------------------------------------------------------------------------------------------------
	void ChangeState( eCustomerState state )
	{
		ExitState(m_state);
		m_state = state;
		EnterState(m_state);
	}


	//------------------------------------------------------------
	void InLine()
	{
		m_moveTimer += Time.deltaTime * m_speed / 20.0f;

		//Move towards spot
		gameObject.transform.position = Vector3.Lerp(m_startSpot.transform.position, m_targetSpot.transform.position, m_moveTimer);

		if( m_moveTimer >= 1.0f)
		{
			ChangeState(eCustomerState.ORDERING);
		}
	}


	//------------------------------------------------------------
	public void Ordering()
	{
		m_waitTimer += Time.deltaTime;

		//Waiting too long
		if(m_waitTimer > m_timeUntilNextStar)
		{
			m_waitTimer -= m_timeUntilNextStar;
			m_rating -= 1.0f;

			if ( m_rating < 1.0f)
			{
				m_rating = 1.0f;
				ChangeState(eCustomerState.LEAVING);
			}

			UpdateStars();
		}
	}


	//------------------------------------------------------------
	void Leaving()
	{
		m_moveTimer += Time.deltaTime * m_speed / 20.0f;

		//Move towards exit
		gameObject.transform.position = Vector3.Lerp(m_targetSpot.transform.position, m_startSpot.transform.position, m_moveTimer);

		if (m_moveTimer >= 1.0f)
		{
			ChangeState(eCustomerState.DEAD);
		}
	}


	//------------------------------------------------------------
	eDrinkType GetRandomDrink()
	{
		return (eDrinkType)Random.Range(1, (int)eDrinkType.COUNT);
	}


	//------------------------------------------------------------
	void CheckGiven()
	{
        //Wrong drink
        if ( m_request != m_given )
		{
			m_rating = 1.0f;
            GameManager.instance.PlayRandomOuch();
		}
        //Correct drink
        else
        {
            for(int handIndex = 0; handIndex < 2; ++handIndex)
            {
                hands[handIndex].GetComponent<MeshRenderer>().material = goodHandMaterial;
            }
            GameManager.instance.PlayRandomPositiveDing();
        }
		ChangeState(eCustomerState.LEAVING);
	}


	//---------------------------------------------------------------------------------------------------
	void UpdateStars()
	{
		if (m_rating < 5.0f)
		{
			m_stars[4].SetActive(false);
		}
		if (m_rating < 4.0f)
		{
			m_stars[3].SetActive(false);
		}
		if (m_rating < 3.0f)
		{
			m_stars[2].SetActive(false);
		}
		if (m_rating < 2.0f)
		{
			m_stars[1].SetActive(false);
		}
	}


	//------------------------------------------------------------
	public void GiveDrink(eDrinkType drinkType)
	{
		//Bad drink type
		if(drinkType == eDrinkType.COUNT ||
			drinkType == eDrinkType.NONE)
		{
			return;
		}

        if( m_state == eCustomerState.ORDERING)
        {
            if (!m_hasDrink)
		    {
			    m_given = drinkType;
			    m_hasDrink = true;
			    CheckGiven();
		    }
        }
	}


	//------------------------------------------------------------
	private void OnTriggerEnter(Collider other)
	{
		CoffeeCupBehaviour coffeeCup = other.gameObject.GetComponent<CoffeeCupBehaviour>();
		if (coffeeCup)
		{
			GiveDrink(coffeeCup.m_filledWith);
			Destroy(other.gameObject);
		}
	}
}
