﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;


//------------------------------------------------------------
public class CustomerSpawner : MonoBehaviour
{
	public static CustomerSpawner instance;
	public GameObject m_customer;
	List<GameObject> m_allCustomers = new List<GameObject>();

	public bool[] m_spotFilled;
	public Transform m_spawnLocation;
	public Transform[] m_spots;

	public float m_spawnRate; //20.0f;
    public float startSpawnRate;
    public float m_maxSpawnRate = 1.0f;
	public float m_spawnTimer = 4.0f;
    float spawnIncreaseSpeed = 0.5f;


	//---------------------------------------------------------------------------------------------------
	void Start()
	{
		if(!instance)
		{
			instance = this;
		}
		else
		{
			Destroy(gameObject);
		}
	}


	//------------------------------------------------------------
	// Update is called once per frame
	void Update ()
	{
		if(GameManager.instance.GetState() != eGameState.IN_GAME)
		{
			return;
		}

		m_spawnTimer += Time.deltaTime;
		if (m_spawnTimer >= m_spawnRate) 
		{
			m_spawnTimer -= m_spawnRate;
			SpawnCustomer();
		}

		//Delete a dead person
		for (int customerIndex = 0; customerIndex < m_allCustomers.Count; ++customerIndex)
		{
			GameObject currentCustomer = m_allCustomers[customerIndex];
			CustomerBehaviour currentCustomerState = currentCustomer.GetComponent<CustomerBehaviour>();
			if (currentCustomerState.m_state == eCustomerState.DEAD)
			{
				m_allCustomers.RemoveAt(customerIndex);
				Destroy(currentCustomer);
				return;
			}
		}
	}


	//------------------------------------------------------------
	void SpawnCustomer()
	{
		List<int> freeSpots = GetFreeSpots();
		//Need at least one spot
		if(freeSpots.Count > 0)
		{
			int freeIndex = Random.Range(0, freeSpots.Count);
			int spotIndex = freeSpots[freeIndex];
			GameObject temp = Instantiate(m_customer, m_spawnLocation.transform.position, Quaternion.identity);
			CustomerBehaviour customerScript = temp.GetComponent<CustomerBehaviour>();
			customerScript.m_startSpot = m_spawnLocation;
			customerScript.m_targetSpot = m_spots[spotIndex];
			customerScript.m_state = eCustomerState.IN_LINE;
			customerScript.m_spotIndex = spotIndex;
			m_allCustomers.Add(temp);
			m_spotFilled[spotIndex] = true;

            m_spawnRate -= spawnIncreaseSpeed;
            if(m_spawnRate < m_maxSpawnRate)
            {
                m_spawnRate = m_maxSpawnRate;
            }
		}
	}


	//------------------------------------------------------------
	List<int> GetFreeSpots()
	{
		List<int> freeSpots = new List<int>();
		for(int spotIndex = 0; spotIndex < 3; ++spotIndex)
		{
			if (!m_spotFilled[spotIndex])
			{
				freeSpots.Add(spotIndex);
			}
		}
		return freeSpots;
	}

	
	//---------------------------------------------------------------------------------------------------
	public void FreeSpot(int spotIsFree)
	{
		m_spotFilled[spotIsFree] = false;
	}


	//---------------------------------------------------------------------------------------------------
	public void Restart()
	{
		//Clear current customers
		while(m_allCustomers.Count > 0)
		{
			Destroy(m_allCustomers[0]);
			m_allCustomers.RemoveAt(0);
		}

		//Clear all spots
		for(int freeIndex = 0; freeIndex < m_spotFilled.Length; ++freeIndex)
		{
			m_spotFilled[freeIndex] = false;
		}

        //Restart spawn timer
        m_spawnRate = startSpawnRate;
        m_spawnTimer = m_spawnRate - 3.0f;
	}
}
