﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using VRTK;


//------------------------------------------------------------
public class GameManager : MonoBehaviour
{
    bool popupShow = false;
    public AnimationCurve popupCurve;
    float m_popupTimer = 0.0f;
	public static GameManager instance;
	private eGameState m_state = eGameState.INTRODUCTION;
	public float m_score = 5.0f;
	int m_servedTotal = 0;
	public GameObject m_startInstructionsText;
	public GameObject m_gameOverText;
	public GameObject m_ratingText;
    public GameObject m_popup;

	[Header("Sounds")]
	public GameObject pourSounds;
    public GameObject ouchSounds;
    public GameObject positiveDingSounds;

	
	//------------------------------------------------------------
	void Start()
	{
		//Making this object a singleton
		if (instance == null)
		{
			instance = this;
		}
		else
		{
			Destroy(gameObject);
		}
	}


	//------------------------------------------------------------
	void Update ()
	{
        //Update popup
        m_popup.SetActive(popupShow);

        m_popupTimer += Time.deltaTime;
        float popupSize = popupCurve.Evaluate(m_popupTimer);
        m_popup.transform.localScale = new Vector3(popupSize*2.5f, popupSize*0.75f, popupSize);

		if(m_score <= 2.0f && GetState() == eGameState.IN_GAME)
		{
			ChangeState(eGameState.GAME_OVER);
		}

		//Debugging
		if(Input.GetKeyDown(KeyCode.Space))
		{
			StartGame();
		}

		//Update score text
		m_ratingText.GetComponent<TextMesh>().text = "rating\n" + (Mathf.Round(m_score * 10.0f) / 10.0f);
	}


	//------------------------------------------------------------
	public void StartGame()
	{
		if( m_state == eGameState.INTRODUCTION ||
			m_state == eGameState.GAME_OVER)
		{
			ChangeState(eGameState.IN_GAME);
		}
	}


	//------------------------------------------------------------
	void ChangeState( eGameState state )
	{
		ExitState(m_state);
		m_state = state;
		EnterState(m_state);
	}


	//------------------------------------------------------------
	void EnterState(eGameState state)
	{
		switch(state)
		{
			case eGameState.IN_GAME:
                popupShow = false;
				m_score = 5.0f;
				m_servedTotal = 0;
				CustomerSpawner.instance.Restart();
				m_startInstructionsText.SetActive(false);
				m_gameOverText.SetActive(false);
				break;
			case eGameState.GAME_OVER:
                popupShow = false;
				m_gameOverText.GetComponent<TextMesh>().text = "Game Over!\nServed " + m_servedTotal;
				m_startInstructionsText.SetActive(true);
				m_gameOverText.SetActive(true);
				break;
		}
	}


	//------------------------------------------------------------
	void ExitState(eGameState state)
	{
		switch (state)
		{
			case eGameState.IN_GAME:
				CustomerSpawner.instance.Restart();
				break;
		}
	}


	//------------------------------------------------------------
	public void AddScore( float score )
	{
        Debug.Log("Score: " + m_score);
        Debug.Log("Added: " + score);
        //Score has 50% influence
        float influence = 0.5f;
		m_score = m_score * (1 - influence) + score * influence;
		m_servedTotal += 1;
        Debug.Log("New Score: " + m_score);
	}


    //------------------------------------------------------------
    public void PlayRandomPour()
    {
        PlayRandomSound(pourSounds);
    }


    //------------------------------------------------------------
    public void PlayRandomOuch()
    {
        PlayRandomSound(ouchSounds);
    }


    //------------------------------------------------------------
    public void PlayRandomPositiveDing()
    {
        PlayRandomSound(positiveDingSounds);
    }


    //------------------------------------------------------------
    void PlayRandomSound( GameObject soundObject )
	{
		AudioSource[] objectSounds = soundObject.GetComponents<AudioSource>();
		int soundIndex = Random.Range(0, objectSounds.Length);
		objectSounds[soundIndex].Play();
	}


	//------------------------------------------------------------
	public eHandSignal GetSignalFromDrinkRequest(eDrinkType drinkType)
	{
		switch(drinkType)
		{
			case eDrinkType.COFFEE:
				return eHandSignal.CIRCLE;
			case eDrinkType.LATTE:
				return eHandSignal.UP_AND_DOWN;
			case eDrinkType.MILK:
				return eHandSignal.WAVE;
			case eDrinkType.MOCHA:
				return eHandSignal.WRIST_FLICK;
			case eDrinkType.TEA:
				return eHandSignal.ZIG_ZAG;
			default:
				Debug.Log("ERROR: INVALID DRINK TYPE");
				return eHandSignal.COUNT;
		}
	}


	//---------------------------------------------------------------------------------------------------
	public eGameState GetState()
	{
		return m_state;
	}


    public void StartPopup( string content )
    {
        popupShow = true;
        m_popupTimer = 0.0f;
        m_popup.transform.GetChild(0).GetComponent<TextMesh>().text = content;
    }
}
