﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;


//------------------------------------------------------------
public class WaveBehaviour : MonoBehaviour
{
	public GameObject m_leftHand;
	public GameObject m_rightHand;

	public Transform m_leftHandCenter;
	public Transform m_rightHandCenter;

	public CustomerBehaviour m_customerBehaviour;

	eHandSignal m_handSignal;

	float m_timer;

	public AnimationCurve[] m_signalOffsets;
	public Vector3[] m_rotations;


	//------------------------------------------------------------
	void Update ()
	{
		m_timer += Time.deltaTime;
		if (m_timer > 1.0f)
		{
			m_timer -= 1.0f;
		}

		eDrinkType request = m_customerBehaviour.m_request;
		m_handSignal = GameManager.instance.GetSignalFromDrinkRequest(request);
		UpdateLeftHand(m_handSignal);
		UpdateRightHand(m_handSignal);
	}


	//------------------------------------------------------------
	void UpdateLeftHand(eHandSignal signal)
	{
		float x = 0.0f;
		float y = 0.0f;
		Quaternion begin = Quaternion.identity;
		Quaternion end = Quaternion.identity;
		Quaternion rotation = Quaternion.identity;
		switch (signal)
		{
			case eHandSignal.WAVE:
				x = m_signalOffsets[0].Evaluate(m_timer);
				y = m_signalOffsets[1].Evaluate(m_timer);
				begin = Quaternion.Euler(m_rotations[0]);
				end = Quaternion.Euler(m_rotations[1]);
				if (m_timer < 0.5f)
				{
					rotation = Quaternion.Lerp(begin, end, m_timer * 2.0f);
				}
				else
				{
					rotation = Quaternion.Lerp(end, begin, (m_timer - 0.5f) * 2.0f);
				}
				break;
			case eHandSignal.CIRCLE:
				x = m_signalOffsets[2].Evaluate(m_timer);
				y = m_signalOffsets[3].Evaluate(m_timer);
				break;
			case eHandSignal.UP_AND_DOWN:
				x = m_signalOffsets[4].Evaluate(m_timer);
				y = m_signalOffsets[5].Evaluate(m_timer);
				break;
			case eHandSignal.ZIG_ZAG:
				x = m_signalOffsets[6].Evaluate(m_timer);
				y = m_signalOffsets[7].Evaluate(m_timer);
				break;
			case eHandSignal.WRIST_FLICK:
				x = m_signalOffsets[8].Evaluate(m_timer);
				y = m_signalOffsets[9].Evaluate(m_timer);
				begin = Quaternion.Euler(m_rotations[8]);
				end = Quaternion.Euler(m_rotations[9]);
				if (m_timer < 0.5f)
				{
					rotation = Quaternion.Lerp(begin, end, m_timer * 2.0f);
				}
				else
				{
					rotation = Quaternion.Lerp(end, begin, (m_timer - 0.5f) * 2.0f);
				}
				break;

		}
		m_leftHand.transform.position = m_leftHandCenter.transform.position + new Vector3(x*0.1f, y*0.1f, 0.0f);
		m_leftHand.transform.rotation = rotation;
	}


	//------------------------------------------------------------
	void UpdateRightHand(eHandSignal signal)
	{
		float x = 0.0f;
		float y = 0.0f;
		Quaternion begin = Quaternion.identity;
		Quaternion end = Quaternion.identity;
		Quaternion rotation = Quaternion.Euler(0.0f, -180.0f, 0.0f);
		switch (signal)
		{
			case eHandSignal.WAVE:
				x = m_signalOffsets[0].Evaluate(m_timer);
				y = m_signalOffsets[1].Evaluate(m_timer);
				begin = Quaternion.Euler(m_rotations[0] + new Vector3(0, -180, 0));
				end = Quaternion.Euler(m_rotations[1] + new Vector3(0, -180, 0));
				if (m_timer < 0.5f)
				{
					rotation = Quaternion.Lerp(begin, end, m_timer * 2.0f);
				}
				else
				{
					rotation = Quaternion.Lerp(end, begin, (m_timer - 0.5f) * 2.0f);
				}
				break;
			case eHandSignal.CIRCLE:
				x = m_signalOffsets[2].Evaluate(m_timer);
				y = m_signalOffsets[3].Evaluate(m_timer);
				break;
			case eHandSignal.UP_AND_DOWN:
				x = m_signalOffsets[4].Evaluate(m_timer);
				y = m_signalOffsets[5].Evaluate(m_timer);
				break;
			case eHandSignal.ZIG_ZAG:
				x = m_signalOffsets[6].Evaluate(m_timer);
				y = m_signalOffsets[7].Evaluate(m_timer);
				break;
			case eHandSignal.WRIST_FLICK:
				x = m_signalOffsets[8].Evaluate(m_timer);
				y = m_signalOffsets[9].Evaluate(m_timer);
				Vector3 endVec = m_rotations[9];
				endVec.x *= -1;
				begin = Quaternion.Euler(m_rotations[8] + new Vector3(0,0,-180));
				end = Quaternion.Euler(endVec + new Vector3(0, 0, -180));
				if (m_timer < 0.5f)
				{
					rotation = Quaternion.Lerp(begin, end, m_timer * 2.0f);
				}
				else
				{
					rotation = Quaternion.Lerp(end, begin, (m_timer - 0.5f) * 2.0f);
				}
				break;

		}
		m_rightHand.transform.position = m_rightHandCenter.transform.position + new Vector3(x * 0.1f, y * 0.1f, 0.0f);
		m_rightHand.transform.rotation = rotation;
	}
}
